#!/usr/bin/env python
from __future__ import print_function
import os
import sys
import logging
import argparse
from  bam2x import IO,TableIO,DBI
import itertools
def help():
    return "pileup bed according a tss ( or other one nt marker) coordinates"
def set_parser(parser):
    parser.add_argument("--x_start",type=int,dest="x_start",default=-500)
    parser.add_argument("--x_end",type=int,dest="x_end",default=500)
    parser.add_argument("-I","--format",type=str,dest="format",choices=("bed6","bed12"),default="bed12")
    parser.add_argument("-s","--score",dest="pileup_score",action="store_true",default=False,help="pile up the bed score")
    
    
def run(args):
    fin=IO.fopen(args.input,"r")
    out=IO.fopen(args.output,"w")
    p_values=[0.0 for i in xrange(args.x_start,args.x_end)]
    n_values=[0.0 for i in xrange(args.x_start,args.x_end)]
    offset=args.x_start
    length=len(p_values)
    for i in TableIO.parse(fin, args.format):
        if args.pileup_score:
            score=i.score
        else:
            score=1.0
        if i.strand=="+" or i.strand==".":
            for e in i.Exons():
                pos_start=e.start-offset
                pos_end=e.stop-offset
                for j in xrange(pos_start,pos_end):
                    if j<0: continue
                    if j>length-1: break
                    p_values[j]+=score
        elif i.strand=="-":
            for e in i.Exons():
                pos_start=e.start-offset
                pos_end=e.stop-offset
                for j in xrange(pos_start,pos_end):
                    if j<0: continue
                    if j>length-1: break
                    n_values[j]+=score
    for pos,pvalue,nvalue in itertools.izip(xrange(args.x_start,args.x_end),p_values,n_values):
        print("{pos}\t{p}\t{n}".format(pos=pos,p=pvalue,n=nvalue),file=out)


if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())




def compress(seq):
    w=""
    d={"A":"A","C":"C","G":"G","T":"T"}
    i=0
    S=""
    for k in seq:
        if d.has_key(w+k):
            w=w+k
        else:
            d[w+k]=i
            i+=1
            S+=str(d[w])+","
            w=k
    for i in d.keys():
        print i,d[i]
    return S


def complexity(seq):
    w=""
    d={"A":"A","C":"C","G":"G","T":"T"}
    i=0
    S=0
    for k in seq.upper():
        if d.has_key(w+k):
            w=w+k
        else:
            d[w+k]=i
            i+=1
            S+=1
            w=k
    return S


if __name__=="__main__":
    b="ACGATACGGAATT"
    a="ACACACACACACACACACACACACACACACACACAC"
    c="ACGACGACGACGA"
    print a,len(a),complexity(a)
    print b,len(b),complexity(b)
    print c,len(c),complexity(c)






from collections import namedtuple
import itertools
import logging

H_PSL=("matches","misMatches","repMatches","nCount","qNumInsert","qBaseInsert","tNumInsert","tBaseInsert","strand","qName","qSize","qStart","qEnd","tName","tSize","tStart","tEnd","blockCount","blockSizes","qStarts","tStart")
H_VCF=("chr","pos","id","ref","alt","qual","filter","info")
F_VCF=(str,int,str,str,str,int,str,str)
H_BED3=("chr","start","stop")
F_BED3=(str,int,int)
H_BED6=("chr","start","stop","id","score","strand")
F_BED6=(str,int,int,str,float,str)
H_BED12=("chr","start","stop","id","score","strand","cds_start","cds_stop","itemRgb","blockCount","blockSizes","blockStarts")
def int_tuple(x):
    a=[]
    for i in x.strip().strip(",").split(","):
        a.append(int(i))
    return tuple(a)
F_BED12=(str,int,int,str,float,str,int,int,str,int,int_tuple,int_tuple)
class METABED(object):
    @property
    def end(self):
        return self.stop
    def __cmp__(self,other):
        return cmp(self.chr,other.chr) or cmp(self.start,other.start) or cmp(self.stop,other.stop)
    def __str__(self):
        return "\t".join([str(i) for i in self])
    def __len__(self):
        return self.stop-self.start
    def Exons(self):
        if hasattr(self,"blockCount"):
            a=[]
            if self.strand=="-":
                step=-1
                j=self.blockCount-1
            else:
                step=1
                j=0
            for i in range(self.blockCount):
                exon_start=self.start+self.blockStarts[i]
                exon_end=self.start+self.blockStarts[i]+self.blockSizes[i]
                exon_id=self.id+"_Exon_"+str(j+1)
                j+=step
                t=(self.chr,exon_start,exon_end,exon_id,0.0,self.strand)
                a.append(BED6(*t))
            if self.strand=="-":
                return a[::-1]
            else:
                return a
        else:
            l=[]
            if hasattr(self,"id"):
                l.append(self._replace(id=self.id+"_Exon_1"))
            else:
                l.append(self)
            return l
    def Introns(self):
        a=[]
        if hasattr(self,"blockCount"):
            if self.strand=="-":
                step=-1
                j=self.blockCount-2
            else:
                step=1
                j=0
            for i in range(self.blockCount-1):
                intron_start=self.start+self.blockStarts[i]+self.blockSizes[i]
                intron_end=self.start+self.blockStarts[i+1]
                intron_id=self.id+"_Intron_"+str(j+1)
                j+=step
                t=(self.chr,intron_start,intron_end,intron_id,0.0,self.strand)
                a.append(BED6(*t))
            if self.strand=="-":
                return a[::-1]
        return a
    def cdna_length(self):
        '''sum of the length of exons'''
        s=0
        for i in self.Exons():
            s=s+i.stop-i.start
        logging.debug("s=%i",s)
        return s
        
    def upstream(self,bp=1000):
        '''return the $bp bp upstream Bed Class Object'''
        chr=self.chr
        strand=self.strand
        id=self.id+"_up"+str(bp)
        if(self.strand=="+"):
            start=self.start-bp
            stop=self.start
        else:
            start=self.stop
            stop=self.stop+bp
        if (start<0):start=0
        x=[chr,start,stop,id,0,strand]
        return BED6(*x)
    def core_promoter(self,bp=1000,down=500):
        '''return the $bp bp upstream Bed Class Object'''
        chr=self.chr
        strand=self.strand
        id=self.id+"_core_promoter"
        if(self.strand=="+"):
            start=self.start-bp
            stop=self.start+down
        else:
            start=self.stop-down
            stop=self.stop+bp
        if (start<0):start=0
        x=[chr,start,stop,id,0,strand]
        return BED6(*x)
    def downstream(self,bp=1000):
        '''return the $bp bp downstream Bed Class Object'''
        chr=self.chr
        strand=self.strand
        id=self.id+"_down"+str(bp)
        if(self.strand=="+"):
            start=self.stop
            stop=self.stop+bp
        else:
            start=self.start-bp
            stop=self.start
        x=[chr,start,stop,id,0,strand]
        return BED6(*x)
    def tss(self):
        '''return the Bed Object that represent transcription start site, name is geneid_tss'''
        pos=self.stop
        if self.strand=="+":
            pos=self.start
        else:
            pos=self.stop-1  #should minus one or not?
        return BED6(self.chr,pos,pos+1,self.id+"_tss",0,self.strand)
    def tts(self):
        '''return the Bed Object that represent transcription termination site, name is geneid_tss'''
        if self.strand=="+":
            pos= self.stop-1
        else:
            pos=self.start 
        return BED6(self.chr,pos,pos+1,self.id+"_tts",0,self.strand)
    def head(self,bp=2):
        '''return the head 2(default) bp of simple bed'''
        if self.strand=="+":
            return BED6(self.chr,self.start,self.start+bp,self.id+"_head"+str(bp)+"bp",0,self.strand)
        if self.strand=="-":
            return BED6(self.chr,self.stop-bp,self.stop,self.id+"_head"+str(bp)+"bp",0,self.strand)
    def tail(self,bp=2):
        '''return the tail 2(default) bp of simple bed'''
        if self.strand=="+":
            return BED6(self.chr,self.stop-bp,self.stop,self.id+"_tail"+str(bp)+"bp",0,self.strand)
        if self.strand=="-":
            return BED6(self.chr,self.start,self.start+2,self.id+"_tail"+str(bp)+"bp",0,self.strand)
def types(x,TYPES):
    b=[]
    for v,t in itertools.izip(x,TYPES):
        b.append(t(v))
    return tuple(b)
class BED3(namedtuple("BED3",H_BED3),METABED):
    @classmethod
    def _types(cls,x):
        return types(x,F_BED3)
    @property
    def strand(self):
        return "."
    @property
    def id(self):
        return "noname"
    @property
    def score(self):
        return 0.0
class BED6(namedtuple("BED6",H_BED6),METABED):
    @classmethod
    def _types(cls,x):
        return types(x,F_BED6)
class VCF(namedtuple("VCF",H_VCF),METABED):
    @classmethod
    def _types(cls,x):
        return types(x,F_VCF)
    @property
    def start(self):
        return self.pos-1
    @property
    def stop(self):
        return self.pos


class BED12(namedtuple("BED12",H_BED12),METABED):
    def _slice(self,start,end,suffix="sliced"):
        chr=self.chr
        
        if start < self.start: start=self.start
        if end > self.stop: end=self.stop
        strand=self.strand
        id=self.id+"_"+suffix
        score=self.score
        itemRgb=self.itemRgb
        cds_start=max(start,self.cds_start)
        cds_stop=min(end,self.cds_stop)
        blockCount=0
        sliceBlockStarts=[]
        sliceBlockSizes=[]
        for blockStart,blockSize in itertools.izip(self.blockStarts,self.blockSizes):
            exon_start=blockStart+self.start
            exon_stop=blockStart+blockSize+self.start
            slice_start=max(start,exon_start)
            slice_stop=min(end,exon_stop)
            if slice_start < slice_stop:
                blockCount+=1
                sliceBlockStarts.append(slice_start-start)
                sliceBlockSizes.append(slice_stop-slice_start)
        if blockCount==0:
            return None
        else:
            return BED12(chr,start,end,id,score,strand,cds_start,cds_stop,itemRgb,blockCount,tuple(sliceBlockSizes),tuple(sliceBlockStarts))
    def utr5(self):
        if self.strand=="+":
            return self._slice(self.start,self.cds_start,"utr5")
        elif self.strand=="-":
            return self._slice(self.cds_stop,self.stop,"utr5")
        else:
            return None
    def utr3(self):
        if self.strand=="-":
            return self._slice(self.start,self.cds_start,"utr3")
        elif self.strand=="+":
            return self._slice(self.cds_stop,self.stop,"utr3")
        else:
            return None
    def cds(self):
        return self._slice(self.cds_start,self.cds_stop,"cds")
    def __str__(self):
        s=""
        for i,x in enumerate(self):
            if i==10 or i==11:
                for y in x:
                    s+=str(y)+","
                s+="\t"
            else:
                s+=str(x)+"\t"
        return s.strip("\t")
    @classmethod
    def _types(cls,x):
        return types(x,F_BED12)
    @property
    def exon_starts(self):
        return [int(i+self.start) for i in self.blockStarts]

    @property
    def exon_stops(self):
        return [int(i+j+self.start) for i,j in itertools.izip(self.blockStarts,self.blockSizes)]
        
    
class Fragment:
    '''
    Paired End Raw Data
    '''
    def __init__(self,read,mate=None,**kwargs):
        self.reads=[]
        if mate is None:
            self.reads.append(read)
        else:
            if read.is_read1:
                self.reads.append(read)
                self.reads.append(mate)
            else:
                self.reads.append(mate)
                self.reads.append(read)
        if kwargs.has_key("chr"):
            self.chr=kwargs["chr"]
    def __str__(self):
        s=""
        for i,x in enumerate(self.reads):
            s+="READ"+str(i)+"\t"+str(x)+"\n"
        return s
    def toBed12(self,chr="unknown_chr",strand="read2",**dict):
        from bam2x import TableIO
        x=list()
        for i in TableIO.parse(self.reads,"bam2bed12",references=chr,strand=strand,**dict):
            x.append(i)
        return x
  

if __name__=="__main__":
    from bam2x.Tools import translate
    b=("chr1",200,500,"test_gene",0.0,"-",250,450,"0,0,0",2,(100,101),(0,199))
    a=BED12(*b)
    print a
    print a.chr
    print a.end
    for i in a.Exons():
        print i
        for j in translate(a,i):
            print j
    for i in a.Introns():
        print i
        for j in translate(a,i):
            print j
    print a.utr3()
    print a.utr5()
    print a.cds()
    print a.exon_starts[1]
    print a.exon_stops[1]



# Programmer : zhuxp
# Date: 
# Last-modified: 01-14-2014, 14:32:37 EST
BLOCKON=3
BLOCKOFF=2
ON=1
OFF=4

DEFAULT_CID=100  ##code machine id  // classify the codw


NORMAL_CODES_ARRAY_HEADER=3721
COMPRESS_CODES_ARRAY_HEADER=9981

BEDGRAPH_CODES_ARRAY_HEADER=4416



from __future__ import print_function
import os
import sys
import logging
import argparse
from bam2x import TableIO,IO,DBI
from bam2x.Tools import compatible_with_transcript,_translate,get_flank_region,translate_coordinates,gini_coefficient
import math
import logging
def help():
    return "aggregation plot near tss, input is bed file and bam file, output the aggregation plot number ( and the gini coefficient , if the gini coefficient is near to 1 , it indicates that most aggregation are contributed by very few tss)"
def set_parser(parser):
    parser.add_argument("-b","--bam",type=str,dest="bam")
    parser.add_argument('-I','--input_format',dest="format",choices=("bed3","bed6","bed12"),default="bed12",type=str,help="input file format default=%(default)s")
    parser.add_argument("--up",type=int,dest="up",default=500,help="tss upstream bp")
    parser.add_argument("--down",type=int,dest="down",default=500,help="tss downstream bp")
    #parser.add_argument("--strand",type=str,dest="strand",choices=("read1","read2"),default="read1",help="paired end read strand, default: %(default)s")
  
def run(args):
    logging.basicConfig(level=logging.INFO)
    up=args.up
    down=args.down
    bp_num=up+down
    offset=-up
    bam=DBI.init(args.bam,"bam")
    fin=IO.fopen(args.input,"r")
    out=IO.fopen(args.output,"w")
    bin_sum=[0 for i in xrange(bp_num)]
    bin_e=[0.0 for i in xrange(bp_num)]
    bin_dis=[[] for i in xrange(bp_num)]
    for i0,bed in enumerate(TableIO.parse(fin,args.format)):
        bed_bin=[0 for i in xrange(bp_num)]
        tss=bed.tss()
        tss_flank=get_flank_region(tss,up,down)
        for read in bam.query(tss_flank,"bam1",strand="read1"):
            a=translate_coordinates(tss,read)
            #print(a,file=out)
            for e in a.Exons():
                #print(e,file=out)
                start=e.start-offset
                end=e.stop-offset
                if start < 0: start=0
                if end > bp_num: end=bp_num
                for j in xrange(start,end):
                    bed_bin[j]+=1
        for  i in xrange(bp_num):
            bin_sum[i]+=bed_bin[i]
            bin_dis[i].append(bed_bin[i])
    bed_num=i0+1
    for i in xrange(bp_num):
        bin_e[i]=gini_coefficient(bin_dis[i])
    print("pos_to_tss\taggregation_mean\tgini_coefficient",file=out)
    for i in xrange(bp_num):
        print("{bin}\t{aggregation}\t{E}".format(bin=i+offset,aggregation=float(bin_sum[i])/bed_num,E=bin_e[i]),file=out)






if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())




from __future__ import print_function
import os
import sys
import logging
import argparse
from  bam2x import IO,TableIO,DBI
def help():
    return "pileup bed"
def set_parser(parser):
    #parser.add_argument("-m",type=str,choices=("seq","cDNA","cdna","cds","utr5","utr3"),dest="method")
    
    
def run(args):
    fin=IO.fopen(args.input,"r")
    out=IO.fopen(args.input,"w")

if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())










# Programmer : zhuxp
# Date: 
# Last-modified: 03-18-2014, 15:11:40 EDT

import os,sys
from bam2x.Annotation import *
from bam2x.Annotation import BED6 as Bed
from bam2x.TableIO import hclass
from bam2x import TableIO
from bam2x.Struct import binindex
import pysam
from bam2x.Tools import rc
from bam2x import Tools
from twobitreader import *
import csv
from bam2x import IO
'''
BASIC QUERY FUNCTIONS
'''

def parse_region_str(x):
    try:
        a=x.split(":")
        b=[None,None]
        if len(a)==2:
            b=a.split("-")
            b[0]=int(b[0])-1
            b[1]=int(b[1])
        return BED3(a[0],b[0],b[1])
    except:
        print >>sys.stderr,"unknown formatted region string"
        raise
class MetaDBI:
    '''
    Meta Class for all DBI (Database Interface)
    '''
    def __init__(self,data,**dict):
        self.data=data
    def query(self,x):
        pass
    def __iter__(self):
        for i in self.data:
            yield i
class BinIndexI(MetaDBI):
    '''
    DBI for flat annotation file or list
    cls
    '''
    def __init__(self,file,**dict):
        '''
        Wrapped in bam2x.DBI.init()
        BinIndex(file,cls=inherited_namedtuplecls)
        inherited_namedtuplecls should have _make and _types functino
        or
        assuming the entry in container is already formatted
        BinIndex(container) 
        '''
        if isinstance(file,str):
            file=csv.reader(IO.fopen(file,"r"),delimiter="\t")
        if dict.has_key("cls"):
            cls=dict["cls"]
            if isinstance(cls,str):
                if hclass.has_key(cls):
                    cls=hclass[cls]
                else:
                    print >>sys.stderr,"UNKNOWN FORMAT %s IN BININDEX DATA STRUCT"%cls
    
            self.data=binindex(file,cls=cls)
        else:
            self.data=binindex(file)
    def query(self,x=None,**kwargs):
        '''
        yield the overlap features with x.
        '''
        if x is None:
            try:
                x=BED3(kwargs["chr"],kwargs["start"],kwargs["stop"])
            except:
                print >>sys.stderr,"UNKNOWN QUERY"
                raise 
        if isinstance(x,str):
            x=parse_region_str(x)
        for i in self.data.query(x,**kwargs):
            yield i
    def close(self):
        '''
        release memory
        '''
        self.data=None 
        
class TabixI(MetaDBI):
    '''
    New DBI for tabix file. 
    TabixI(filename,cls=inherited_namedtuplecls)
    '''
    def __init__(self, tabix_file_name,**dict):
        '''
        wrapped in DBI.init(filename,"tabix",cls=cls)
        '''
        self.tabix_file_name=tabix_file_name
        self.dict=dict
        
        if dict.has_key("cls"):
            self.cls=dict["cls"]
            if isinstance(self.cls,str):
                if hclass.has_key(self.cls):
                    self.cls=hclass[self.cls]
                else:
                    print >>sys.stderr,"UNKNOWN FORMAT %s IN BININDEX DATA STRUCT"%cls
        try:
            self.data=pysam.Tabixfile(tabix_file_name)
        except:
            print >>sys.stderr,"WARNING: Can't init the tabix file",tabix_file_name
    def query(self,x=None,**kwargs):
        '''
        yield the overlap feature in tabix index files
        
        dbi=TabixI(filename,cls=cls)
        three options:
        dbi.query(x)  , x is BED class or any class have .chr, .start, .stop
        dbi.query(chr="",start="",stop="")
        dbi.query("chr01:1-10000")
        '''
        try:
            if x is None:
                try:
                    x=BED3(kwargs["chr"],kwargs["start"],kwargs["stop"])
                except:
                    print "UNKNOWN QUERY"
                    raise 
            if isinstance(x,str):
                x=parse_region_str(x)
            if hasattr(self,"cls"):
                for item in self.data.fetch(x.chr,x.start,x.stop):
                    item=self.cls._make(self.cls._types(item.split("\t")))
                    yield item
            else:
                for item in self.data.fetch(x.chr,x.start,x.stop):
                    yield item

        except Exception as e:
           print e
           raise
           raise StopIteration
    def close(self):
        self.data.close()
class TwoBitI(MetaDBI): 
    def __init__(self,File,**kwargs):
        '''
        query 2 bit genome
        '''
        self.data=TwoBitFile(File)
    def query(self,x):
        chr=self.data[x.chr]
        return chr[x.start:x.stop]
class GenomeI(TwoBitI):
    '''
    Wrapped for query sequence
    Initialize genome 2bit file only once.
    '''
    def query(self,x=None,**dict):
        if x is None:
            try:
                x=BED3(kwargs["chr"],kwargs["start"],kwargs["stop"])
            except:
                print "UNKNOWN QUERY"
                raise 
        if isinstance(x,str):
            x=parse_region_str(x)
        method="seq"
        if(dict.has_key("method")):
            method=dict["method"]
        if method=="seq":
            return self.get_seq(x)
        elif method=="cdna" or method=="cDNA":
            return self.get_cdna_seq(x)
        elif method=="cds" or method=="CDS":
            return self.get_cds_seq(x)
        elif method=="utr3":
            return self.get_utr3_seq(x)
        elif method=="utr5":
            return self.get_utr5_seq(x)
    def get_seq(self,x):
        chr=self.data[x.chr]
        seq=chr[x.start:x.stop]
        if x.strand=="-":
            seq=rc(seq)
        return seq
    def get_cdna_seq(self,bed12):
        s=""
        for i in bed12.Exons():
            s+=self.get_seq(i)
        return s
    def get_cds_seq(self,bed12):
        cds=bed12.cds()
        if cds is None or len(cds)==0: return ""
        return self.get_cdna_seq(cds)
    def get_utr3_seq(self,bed12):
        utr3=bed12.utr3()
        if utr3 is None or len(utr3)==0: return ""
        return self.get_cdna_seq(utr3)
    def get_utr5_seq(self,bed12):
        utr5=bed12.utr5()
        if utr5 is None or len(utr5)==0: return ""
        return self.get_cdna_seq(utr5)        



class BamlistI(MetaDBI):
    '''
    A DBI for a list of bamfiles ( or a file contain bamfile names) 
    Query a Bed Object (or Annotation Format inherit Bed)
    Yield Nt Distribution from the start position to the end position
    Nt Distribution [A,C,G,T]
    Example:
        [0,12,0,23]  which means  12 C and 23 T 
    Query Example:
        from bam2x import DBI

        bamfiles=["file1.bam","file2.bam","file3.bam"]
        dbi=DBI.init(bamfiles,"bamlist")
        bed=Bed(["chr1",0,5,".",".","."])
        for i,x in enumerate(dbi.query(bed)):
            print "position",i+1,"\t",x

        the output example:
        position 1   [0,1,4,0]
        position 2   [0,0,0,3]
        .
        .
        .
        
        Which indicates that 

        1 C 4 G in position 1
        3 T in position 2 

        ....
        in all file1.bam, file2.bam and file3.bam
        
    '''

    hNtToNum={'a':0,'A':0,
          'c':1,'C':1,
          'g':2,'G':2,
          't':3,'T':3
         }
    Nt=['A','C','G','T']
    
    
    def __init__(self,bamfiles,**dict):
        '''
        '''
        if type(bamfiles)==type("string"):
            filename=bamfiles
            bamfiles=[]
            for i in TableIO.parse(filename,"simple"):
                bamfiles.append(i[0])
        self.bamfiles=[]
        for bamfile in bamfiles:
            if type(bamfile)==type("str"):
                try:
                    bamfile=pysam.Samfile(bamfile,"rb")
                except:
                    print >>sys.stderr,"WARNING: Can't init the bam file",bamfile
            self.bamfiles.append(bamfile)
    def close(self):
        for i in self.bamfiles:
            i.close()
    @property
    def mapped(self):
        s=0
        for i in self.bamfiles:
            s+=i.mapped
        return s

    def query(self,x=None,method='fetch',**dict):
        if x is None:
            try:
                x=BED3(dict["chr"],dict["start"],dict["stop"])
            except:
                print "UNKNOWN QUERY"
                raise 
        if type(x)==type("str"):
            x=parse_region_str(x)
        chrom=x.chr
        start=x.start
        end=x.stop
        
        uniq=False
        if dict.has_key("uniq"):
            uniq=dict["uniq"]
        if method=='fetch' or method=="bam2bed12" or method=="fetch12":
            '''
            test version
            still test Tools.cigar_to_coordinates
            '''
            for bamfile in self.bamfiles:
                for bed in TableIO.parse(bamfile.fetch(chrom,start,end),"bam2bed12",references=chrom,uniq=uniq):
                    yield bed
        elif method=="bam1": 
        # fetch read from paired end with strand information  
            for bamfile in self.bamfiles:
                strand="read2"
                if dict.has_key("strand"):   # TODO: if bamfiles have different read1 or read2 ?
                    strand=dict["strand"]
                for bed in TableIO.parse(bamfile.fetch(chrom,start,end),"bam2bed12",references=chrom,strand=strand,uniq=uniq):
                    yield bed
        elif method=="bam2": # yield bed12
            for bamfile in self.bamfiles:
                for fragment in TableIO.parse(bamfile.fetch(chrom,start,end),"bam2fragment",bam=bamfile):
                    if dict.has_key("strand"):
                        frag=fragment.toBed12(chr=chrom,strand=dict["strand"],uniq=uniq)
                        if frag: yield frag
                    else:
                        frag=fragment.toBed12(chr=chrom,uniq=uniq)
                        if frag: yield frag
        elif method=="bam2fast": # yield bed12 , don't report mate not in this iterator, 
            for bamfile in self.bamfiles:
                for fragment in TableIO.parse(bamfile.fetch(chrom,start,end),"bam2fragment"):
                    if dict.has_key("strand"):
                        frag=fragment.toBed12(chr=chrom,strand=dict["strand"],uniq=uniq)
                        if frag: yield frag
                    else:
                        frag=fragment.toBed12(chr=chrom,uniq=uniq)
                        if frag: yield frag
        elif method=='pileup':
            s=[[0,0,0,0] for row in range(end-start)]
            for bamfile in self.bamfiles:
                try:
                    A=bamfile.pileup(chrom,start,end)
                except:
                    print >>sys.stderr,"Can't pile up",chrom,start,end
                    raise StopIteration 
                for pileupcolumn in A:
                    j=pileupcolumn.pos-start
                    if j<0: continue
                    if j>end-start: break
                    for pileupread in pileupcolumn.pileups:
                        try:
                            if pileupread.is_del: continue
                            if pileupread.indel!=0: continue
                            nt=pileupread.alignment.seq[pileupread.qpos]
                            if BamI.hNtToNum.has_key(nt):
                                k=BamI.hNtToNum[nt]
                                s[j][k]+=1
                        except:
                            pass
            for i in s:
                yield i
        elif method=='count':
            s=0
            for bamfile in self.bamfiles:
                s+=bamfile.count(chrom,start,end)
            yield s
        elif method=='count_fragment':
            s=0
            for bamfile in self.bamfiles:
               for fragment in TableIO.parse(bamfile.fetch(chrom,start,end),"bam2fragment",bam=bamfile):
                   s+=1
            yield s
        elif method=="seq_and_qual":
            for bamfile in self.bamfiles:
                for read in bamfile.fetch(chrom,start,end):
                    yield (read.seq,read.qual)
        

class BamI(BamlistI):
    '''
    A DBI for bamfile
    Query a Bed Object (or Annotation Format inherit Bed)
    Yield Nt Distribution from the start position to the end position

    It is an simple version for BamlistI.
    Use it to  query one BamFile

    Wrapped in DBI.query(bamfilename,"bam")

    Query Example:
        from bam2x import DBI

        bamfiles="file1.bam"
        dbi=DBI.init(bamfiles,"bam")
        bed=Bed(["chr1",0,5,".",".","."])
        for i,x in enumerate(dbi.query(bed)):
            print "position",i+1,"\t",x
    '''
    def __init__(self,bamfile,**dict):
        '''
        init Bam 
        convert filename bamfile into [bamfile].
        '''
        self.bamfiles=[]
        bamfiles=[bamfile]
        for bam in bamfiles:
            if type(bam)==type("str"):
                try:
                    bam=pysam.Samfile(bamfile,"rb")
                except:
                    print >>sys.stderr,"WARNING: Can't init the bam file",bam
            self.bamfiles.append(bam)
 


class BigWigI(MetaDBI):
    '''
    A DBI for bigwig file
    lib dependent: bxpython
    '''
    def __init__(self,bwfile,**dict):
        '''
        init bw file
        '''
        try:
            import bx.bbi.bigwig_file
        except ImportError:
            raise
        if type(bwfile)==type("string"):
            self.data=bx.bbi.bigwig_file.BigWigFile(open(bwfile,"rb"))
        else:
            try:
                self.data=bx.bbi.bigwig_file.BigWigFile(bwfile)
            except:
                print >>sys.stderr,"Error in open bw file"

    def query(self,x=None,**dict):
        '''
        query bw file
        '''
        if x is None:
            try:
                x=BED3(dict["chr"],dict["start"],dict["stop"])
            except:
                print "UNKNOWN QUERY"
                raise 
        if type(x)==type("str"):
            x=parse_region_str(x)
 
        if not dict.has_key("method"):
            results=self.data.get_as_array(x.chr,x.start,x.stop)
            if hasattr(x,"strand") and x.strand=="-":
                return results[::-1]
            else:
                return results
        else:
            if dict["method"]=="cDNA" or dict["method"]=="cdna":
                s=[]
                for i in x.Exons():
                    array=self.data.get_as_array(i.chr,i.start,i.stop)
                    size=i.stop-i.start
                    if  x.strand=="+" or x.strand==".":
                        for j in range(size):
                            s.append(array[j])
                    elif x.strand=="-":
                        for j in xrange(size-1,-1,-1):
                            s.append(array[j])
                return s
            else:
                print >>sys.stderr,"query model is not a BED12 class"

        







from __future__ import print_function
import os
import sys
import logging
import argparse
from bam2x import TableIO,IO,DBI
from bam2x.Tools import compatible_with_transcript,_translate
import math
import logging
def help():
    return "aggregation plot, input is bed12 file and bam file, output the aggregation plot number ( and the entropy )"
def set_parser(parser):
    parser.add_argument("-b","--bam",type=str,dest="bam")
    parser.add_argument("-B","--bin_num",type=int,default=100,dest="bin_num")
    
    
def run(args):
    logging.basicConfig(level=logging.INFO)
    bam=DBI.init(args.bam,"bam")
    fin=IO.fopen(args.input,"r")
    out=IO.fopen(args.output,"w")
    bin_sum=[0 for i in xrange(args.bin_num)]
    bin_e=[0.0 for i in xrange(args.bin_num)]
    #neg_bin_sum=[0 for i in xrange(args.bin_num)]
    bin_dis=[[] for i in xrange(args.bin_num)]
    #neg_bin_dis=[[] for i in xrange(args.bin_num)]
    for i0,bed in enumerate(TableIO.parse(fin,"bed12")):
        bed_bin=[0 for i in xrange(args.bin_num)]
        for read in bam.query(bed,"bam1"):
            if compatible_with_transcript(read,bed):
                gene_bed=_translate(bed,read)
                cdna_length=bed.cdna_length()
                bin_start=gene_bed.start*args.bin_num/cdna_length
                bin_stop=gene_bed.stop*args.bin_num/cdna_length
                for i in xrange(bin_start,bin_stop):
                    bed_bin[i]+=1
        for i in xrange(args.bin_num):
            bin_sum[i]+=bed_bin[i]
            bin_dis[i].append(bed_bin[i])
        if i0%10==0:
            logging.info("processing %d gene",i0)
    for i in xrange(args.bin_num):
        bin_e[i]=dis2entropy(bin_dis[i])
    print("bin\taggregation\tentropy",file=out)
    for i in xrange(args.bin_num):
        print("{bin}\t{aggregation}\t{E}".format(bin=i,aggregation=bin_sum[i],E=bin_e[i]))




def dis2entropy(iterator):
    s=0
    h={}
    for i in iterator:
        if h.has_key(i):
            h[i]+=1
        else:
            h[i]=1
        s+=1
    e=0.0
    for i in h.values():
        f=float(i)/s
        if f!=0.0:
            e+=f*math.log(f)
    return e


if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())




import string
from bam2x.Annotation import BED12,BED6,BED3
bed12_schema_template= string.Template("""create table if not exists $table_name (
    id      integer primary key autoincrement not null,
    chr     varchar(20),
    start   integer,
    stop    integer,
    name    varchar(200),
    score   float,
    strand  character(1),
    cds_start   integer,
    cds_stop    integer,
    itemRgb     varchar(20),
    blockCount  integer,
    blockSizes  varchar,
    blockStarts varchar
)
""")

bed12_insert_template=string.Template("""insert into $table_name (chr,start,stop,name,score,strand,cds_start,cds_stop,itemRgb,blockCount,blockSizes,blockStarts)
                            values (?,?,?,?,?,?,?,?,?,?,?,?)
                            """)
 
bed12_select_template=string.Template("""select * from $table_name where name==$name""")


bed6_schema_template= string.Template("""create table if not exists $table_name (
    id      integer primary key autoincrement not null,
    chr     varchar(20),
    start   integer,
    stop    integer,
    name    varchar(200),
    score   float,
    strand  character(1)
)
""")

bed3_schema_template= string.Template("""create table if not exists $table_name (
    id      integer primary key autoincrement not null,
    chr     varchar(20),
    start   integer,
    stop    integer
)
""")

bed6_insert_template=string.Template("""insert into $table_name (chr,start,stop,name,score,strand)
                            values (?,?,?,?,?,?)
                            """)
 

bed3_insert_template=string.Template("""insert into $table_name (chr,start,stop)
                            values (?,?,?)
                            """)
 

schema_templates={
    "bed12":bed12_schema_template,
    "bed6":bed6_schema_template,
    "bed3":bed3_schema_template,
}
insert_templates={   
    "bed12":bed12_insert_template,
    "bed6":bed6_insert_template,
    "bed3":bed3_insert_template,
}

factories = {
"bed12":lambda c,r:BED12._make(BED12._types(r[1:])),
"bed6":lambda c,r:BED6._make(BED6._types(r[1:])),
"bed3":lambda c,r:BED3._make(BED3._types(r[1:])),
}



select_template=string.Template("""select * from $table_name where name=\"$name\"""")



#!/usr/bin/env python
# Programmer : zhuxp
# Date: 
# Last-modified: 02-18-2014, 12:54:33 EST

from bam2x.Annotation import BED12
from bitarray import bitarray
import TuringCodeBook as cb


def twobitarray_and(bitarrayA,bitarrayB):
    '''
    '11' code unknown
    '10' code block on
    '01' code block off

    usage:
    twobitarray_and(a,b)

    return true if they are compatible  
    which means there no '00' in each twobit and operation
    '''
    l1=len(bitarrayA)
    l2=len(bitarrayB)
    l=l1
    if l>l2: l=l2
    for i in range(0,l,2):
        a0=bitarrayA[i]
        a1=bitarrayA[i+1]
        b0=bitarrayB[i]
        b1=bitarrayB[i+1]
        if (a0!=b0 and a1!=b1): return False
    return True

def bitarray_and(bitarrayA,bitarrayB):
    l1=len(bitarrayA)
    l2=len(bitarrayB)
    if l1!=l2: return None
    C=bitarray(l1)
    for i in range(l1):
        C[i]=bitarrayA[i] and bitarrayB[i]
    return C

def bitarray_to_intron_number(bitarray):
    l=len(bitarray)
    s=0
    state=0
    for i in range(0,l,2):
        if bitarray[i] and (not bitarray[i+1]):
            state=1
        if (not bitarray[i]) and bitarray[i+1]:
            if state!=-1:
                s+=1
            state=-1
    return s    

def bitarray_to_rep(bitstr):
    s=""
    for i in range(0,len(bitstr),2):
        if bitstr[i] and (not bitstr[i+1]):
            s+="E"
        elif (not bitstr[i]) and bitstr[i+1]:
            s+="i"
        elif not bitstr[i] and not bitstr[i+1]:
            s+="x"
        else:
            s+="_"
    return s
def bitarray_codes_to_rep(codes,bitstr):
    s=""
    pos={}
    for i in codes:
        if not pos.has_key(i[0]):
            pos[i[0]]=1
    keys=pos.keys()
    keys.sort()
    for i in range(0,len(bitstr),2):
        if bitstr[i] and (not bitstr[i+1]):
            s+=str(keys[i/2])+"E "
        elif (not bitstr[i]) and bitstr[i+1]:
            s+=str(keys[i/2])+"i "
        elif not bitstr[i] and not bitstr[i+1]:
            s+=str(keys[i/2])+"x "
        else:
            s+=str(keys[i/2])+"_ "
    return s
def TuringFactory(bed12):
    import bam2x.Turing.TuringCodeBook as cb
    from bam2x.Turing import TuringCode, TuringGraph
    g=[]
    g.append(TuringCode(bed12.start,cb.ON))
    g.append(TuringCode(bed12.stop,cb.OFF))
    if isinstance(bed12,BED12):
        for i in bed12.Exons():
            g.append(TuringCode(i.start,cb.BLOCKON))
            g.append(TuringCode(i.stop,cb.BLOCKOFF))
    g.sort()
    return TuringGraph(g)

def count_xor(path):
    s=0
    for i in range(0,len(path),2):
        if path[i]^path[i+1]: s+=1 # ^ is xor 
    return s        

def isCompatible(pathA,pathB):  ##pathA and B shoulc be same length
    #print "debug pathA",pathA
    #print "debug pathB",pathB
    for i in range(0,len(pathA),2):
        if not (pathA[i]&pathB[i] | pathA[i+1]&pathB[i+1]):
            return False
    return True
def isSelfIncompatible(pathA):
    #print "debug",pathA
    for i in range(0,len(pathA),2):
        if not pathA[i]:
            if not pathA[i+1]:
                return True
    return False

def isOverlap(pathA,pathB):   ## Fir paired end reads overlap is not start > stop, overlap definition is not the same as overlap reades
    for i in range(0,len(pathA),2):
        if (pathA[i]^pathA[i+1]) and (pathB[i]^pathB[i+1]):
            return True
    return False


def isOverlapCompatible(pathA,pathB):
    return isOverlap(pathA,pathB) and isCompatible(pathA,pathB)



# Programmer : zhuxp
# Date:  Sep 2012
# Last-modified: 03-26-2014, 11:15:01 EDT
from string import upper,lower
from bam2x.Annotation import BED6 as Bed
from bam2x.Annotation import BED12 as Bed12
from bam2x.Annotation import BED12 
from bam2x.Annotation import BED3
from bam2x.Annotation import Fragment
import bam2x
import copy
import logging
import itertools
# __all__=["IO","codon"]

hNtToNum={'a':0,'A':0,
          'c':1,'C':1,
          'g':2,'G':2,
          't':3,'T':3
         }
Nt=['A','C','G','T']

suffixToFormat={
    'fa':'fasta',
    'fq':'fastq',
    'genetab':'genebed',
    'bw':'bigwig',
    'tab':'genebed',
    '2bit':'genome'

}

def get_flank_region(bed,up,down,chr_lengths=None):
    down_bp=down
    up_bp=up
    if bed.strand=="-":
        start=bed.start-down
        stop=bed.stop+up
        if start < 0:
            start=0
            down_bp=bed.start
        if chr_lengths is not None and chr_lengths.has_key(bed.chr):
            if stop > chr_lengths[bed.chr]:
                stop=chr_lengths[bed.chr]
                up_bp=stop-bed.stop
    elif bed.strand=="+" or bed.strand==".":
        start=bed.start-up
        stop=bed.stop+down
        if start < 0:
            start=0
            up_bp=bed.start
        if chr_lengths is not None and chr_lengths.has_key(bed.chr):
            if stop > chr_lengths[bed.chr]:
                stop=chr_lengths[bed.chr]
                down_bp=stop-bed.stop
    return bed._replace(id="{id}_up{up}_down{down}".format(id=bed.id,up=up_bp,down=down_bp),start=start,stop=stop)
    
def rc(seq):
   comps = {'A':"T", 'C':"G", 'G':"C", 'T':"A",
           'B':"V", 'D':"H", 'H':"D", 'K':"M",
           'M':"K", 'R':"Y", 'V':"B", 'Y':"R",
           'W':'W', 'N':'N', 'S':'S'}
   return ''.join([comps[x] for x in seq.upper()[::-1]])
def shuffle(seq):
   import random
   a=list(seq)
   random.shuffle(a)
   return "".join(a)

def seq_wrapper(seq,width=60):
    s=""
    seqlen=len(seq)
    for i in range(0,seqlen,width):
        stop=i+width
        if stop>seqlen:stop=seqlen
        s+=seq[i:stop]+"\n"
    return s
def distance(A,B):
    if A.chr!=B.chr: return None
    if overlap(A,B): return 0
    return min(abs(A.start-B.stop),abs(A.stop-B.start))
def translate_coordinate(coord,bed,reverse=False):
    '''
    translate bed's coordiante based on coord
    if reverse is True:
        bed is in coord's coordinates and translate it back to the chromosome coordinates
    coord is a simple BED6 or BED3, didn't consider the splicing in this function
    '''
    if reverse:
        #TO TEST
        chr=coord.chr
        id=bed.id
        if coord.strand=="+" or coord.strand==".":
            return (coord.start+bed.start,coord.start+bed.stop,bed.strand)
        else:
            strand="."
            if bed.strand=="-":strand="+"
            if bed.strand=="+":strand="-"
            return (coord.stop-bed.stop,coord.stop-bed.start,strand)

    else:
        if coord.chr!=bed.chr: return None
        if coord.strand=="+" or coord.strand==".":
            return (bed.start-coord.start,bed.stop-coord.start,bed.strand)
        if coord.strand=="-":
            strand="."
            if bed.strand=="-":strand="+"
            if bed.strand=="+":strand="-"
            return (coord.stop-bed.stop,coord.stop-bed.start,strand)
def translate_coordinates(coord,bed,reverse=False): # bed is Bed12 format
    '''
    Translate Bed12 Object bed to BED3 or BED6 coordinates ( no splicing )
    if reverse is True
        bed is in coord's coordinates and translate it back to the chromosome coordinates.
        TO TEST
    '''
    if reverse:
        chr=coord.chr
    else:
        chr=coord.id
    id=bed.id
    (start,stop,strand)=translate_coordinate(coord,bed,reverse)
    score=bed.score
    if isinstance(bed,Bed12):
        (cds_start,cds_stop,cds_strand)=translate_coordinate(coord,Bed(bed.chr,bed.cds_start,bed.cds_stop,bed.id+"_cds",bed.score,bed.strand),reverse)
        itemRgb=bed.itemRgb
        blockCount=bed.blockCount
        blockSizes=list(bed.blockSizes)
        blockStarts=list(bed.blockStarts)
        
        if coord.strand=="+" or coord.strand==".":
            for i,x in enumerate(blockStarts):
                blockStarts[i]=blockStarts[i]
        elif coord.strand=="-":
            for i,x in enumerate(blockStarts):
                #print "debug",i,x,blockStarts,blockSizes
                blockStarts[i]+=blockSizes[i]
            blockStarts=blockStarts[::-1]
            for i,x in enumerate(blockStarts):
                #print blockStarts[i]
                blockStarts[i]=bed.stop-(blockStarts[i]+bed.start)
            blockSizes=blockSizes[::-1]
        return Bed12(chr,start,stop,id,score,strand,cds_start,cds_stop,itemRgb,blockCount,tuple(blockSizes),tuple(blockStarts))
    else:
        C=copy.copy(bed)
        C.chr=chr
        C.start=start
        C.stop=stop
        C.end=stop
        C.strand=strand
        if bed.__dict__.has_key('pos'):
            C.pos=C.start+1  # position is 1  index
        return C
        
def merge_bed(bedA,bedB,id="noname"):
    '''
    merge two bed into a bed , ratain all exon region.
    '''
    if bedA.chr!=bedB.chr: return None
    l=[]
    for i in bedA.Exons():
        l.append(i)
    for i in bedB.Exons():
        l.append(i)
    return _merge_bed6(l,id=id)
    #TODO 
def _merge_bed6(beds,id="noname"):
    '''
    a simple turing state change
    '''
    l=[]
    strand=beds[0].strand
    for i in beds:
        l.append((i.start,-1))
        l.append((i.stop,1))
        if strand!=i.strand: strand="."
    chr=beds[0].chr
    l.sort()
    switch=0
    
    start=l[0][0]
    end=l[-1][0]
    
    state=0
    assert i[0][1] > 1
    last_pos=l[0][0]
    state=1
    switch=1
    blockStarts=[]
    blockSizes=[]
    for i in l[1:]:
        state-=i[1]
        if switch==1 and state==0:
            blockStarts.append(last_pos-start)
            blockSizes.append(i[0]-last_pos)
            switch=0
        if switch==0 and state>0:
            last_pos=i[0]
            switch=1
    
    assert state==0
    blockCount=len(blockSizes)
    return BED12(chr,start,end,id,0.0,strand,start,start,"0,0,0",blockCount,blockSizes,blockStarts) 

def translate(bedA,bedB):
    #TODO
    '''
    INPUT: two BED12 bedA and bedB 
    return the location for bedA in merge bed, the location for bedB in merge bed, merged bed
    '''
    meta=merge_bed(bedA,bedB,bedA.id+"_"+bedB.id+"_merged")
    new_bedA=_translate_to_meta(meta,bedA)
    new_bedB=_translate_to_meta(meta,bedB)
    return new_bedA,new_bedB,meta
def _translate_to_meta(meta,bed):
    '''
    '''
    l=[]
    for i in meta.Exons():
        l.append((i.start,-1,1))
        l.append((i.stop,1,1))
    for i in bed.Exons():
        l.append((i.start,-1,2))
        l.append((i.stop,1,2))
    if hasattr(bed,"cds_start"):
        l.append((bed.cds_start,0,3))
        l.append((bed.cds_stop,0,4))
    else:
        '''
        no cds 
        '''
        l.append((bed.start,0,3))
        l.append((bed.start,0,4))

    l.sort()
    meta_state=0
    bed_state=0
    meta_last_pos=0
    meta_coordinate=0
    tl=[]
    for i in l:
        if i[2]==1: #META
            if i[1]==-1 and meta_state==0:
                meta_state=1
                meta_last_pos=i[0]
            elif i[1]==1 and meta_state==1:
                meta_state=0
                meta_coordinate+=i[0]-meta_last_pos 
                meta_last_pos=i[0]
        elif i[2]==2:  #BED
            tl.append((i[0]-meta_last_pos+meta_coordinate,i[1]))
        elif i[2]==3:
            cds_start=i[0]-meta_last_pos+meta_coordinate
        elif i[2]==4:
            cds_stop=i[0]-meta_last_pos+meta_coordinate
    tl.sort()
    state=0
    switch=0
    blockStarts=[]
    blockSizes=[]
    for i in tl:
        state-=i[1]
        if switch==0:
            if state > 0:
                b_start=i[0]
                switch=1
                blockStarts.append(b_start)
        elif switch==1:
            if state == 0:
                b_end=i[0]
                switch=0
                blockSizes.append(b_end-b_start)
    blockCount=len(blockSizes)
    if meta.strand=="." or meta.strand=="+":
        return BED12(meta.id,blockStarts[0],blockStarts[-1]+blockSizes[-1],bed.id,0.0,bed.strand,cds_start,cds_stop,"0,0,0",blockCount,blockSizes,blockStarts)
    else:
        len_meta=meta.cdna_length()
        strand=reverse_strand(bed.strand)
        return BED12(meta.id,len_meta-blockStarts[-1]-blockSizes[-1],len_meta-blockStarts[0],bed.id,0.0,strand,len_meta-cds_stop,len_meta-cds_start,"0,0,0",blockCount,blockSizes[::-1],[len_meta-i0-j0 for i0,j0 in itertools.izip(blockStarts[::-1],blockSizes[::-1])])
    
    
_translate=_translate_to_meta

def reverse_translate(meta,bed):
    '''
    reverse translate bed in meta coordinates ( a BED12 object ) to chromosome coordinates
    '''
    assert meta.id==bed.chr
    start=bed.start
    stop=bed.stop
    start_sign=True
    stop_sign=True

    if meta.strand=="-":
        for blockSize,blockStart in itertools.izip(meta.blockSizes[::-1],[ i0+j0 for i0,j0 in itertools.izip(meta.blockStarts[::-1],meta.blockSizes[::-1])]):
            start-=blockSize
            if start <= 0 and start_sign:
                new_stop=blockStart-blockSize-start+meta.start
                start_sign=False
            stop-=blockSize
            if stop <= 0 and stop_sign:
                new_start=blockStart-blockSize-stop+meta.start
                stop_sign=False
                break
    else:
        for blockSize,blockStart in itertools.izip(meta.blockSizes,meta.blockStarts):
            start-=blockSize
            stop-=blockSize
            if start <= 0 and start_sign:
                new_start=blockStart + start + blockSize + meta.start
                start_sign=False
            if stop  <= 0 and stop_sign:
                new_stop=blockStart + stop + blockSize + meta.start
                stop_sign=False
                break
    return meta._slice(new_start,new_stop,bed.id)






def overlap(A,B):
    '''
    if A is overlapping with B.
    A and B are ? extends Bed class.
    '''
    if(A.chr != B.chr) : return 0
    if (A.stop < B.start) : return 0
    if (B.stop < A.start) : return 0
    return 1
def find_nearest(bed,dbi,extends=50000,**dict):
    start=bed.start-extends
    stop=bed.stop+extends
    chr=bed.chr
    if start<0: start=0
    new_bed=BED3(chr,start,stop)

    results=dbi.query(new_bed,**dict)
    d=2*extends
    flag=0
    
    for result in results:
        d0=distance(bed,result)
        if d0<d:
            d=d0
            nearest=result
            if  result.strand=="." or bed.strand==".":
                strand="."
            elif result.strand==bed.strand:
                strand="+"
            else:
                strand="-"
            flag=1
    if flag==0:
        return (None,None,None)
    else:
        return (d,nearest,strand)





def compatible(a,b,**kwargs):
    '''
    VERSION: TEST
    if a and b are compatible return true
    a and b are BED12 class 
    definition of compatible
       the overlap region should be same transcript structure.
    '''
    if (not overlap(a,b)): return True; 
    if (a.strand!=b.strand): return False;
    '''
    if two bed is not overlap, they are compatible.
    '''
    start=a.start;
    if (start < b.start): start=b.start 
    '''
    start is the max start of a.start and b.start 
    '''
    stop=a.stop
    if (stop > b.stop): stop=b.stop
    '''
    stop is the min stop of a.stop and b.stop
    find the overlap region from [start,stop)
    '''
    sliced_a=a._slice(start,stop)
    sliced_b=b._slice(start,stop)
    if sliced_a is None: return False
    if sliced_b is None: return False
    if sliced_a.blockCount!=sliced_b.blockCount : 
        return False
    else:
        for i in range(sliced_a.blockCount):
            if sliced_a.blockStarts[i]!=sliced_b.blockStarts[i]:
                return False
            if sliced_a.blockSizes[i]!=sliced_b.blockSizes[i]:
                return False
    return True
def reverse_strand(i):
    if i=="+": return "-"
    if i=="-": return "+"
    if i==".": return "."
    if type(i)==type(1): return -i
def compatible_with_transcript(read,transcript,**kwargs): 
    '''
    if reads is compatible with the transcript( full length or longer than reads)
    
    !!! if read is fragment, then need add "references"= bamfile.references
    '''
    # read must in the transcript
    if isinstance(read,Fragment):
        for i0,i in enumerate(bam2x.TableIO.parse(read.reads,"bam2bed12",**kwargs)):
            if i.start < transcript.start or i.stop > transcript.stop :
                return False
            else:
                if kwargs.has_key("strand"): 
                    if kwargs["strand"]=="read1":
                        if read.reads[i0].is_read2:
                            i.strand=reverse_strand(i.strand)
                    elif kwargs["strand"]=="read2":
                        if read.reads[i0].is_read1:
                            i.strand=reverse_strand(i.strand)
                if not compatible(i,transcript,**kwargs):
                    return False
        return True
    else:
        if kwargs.has_key("strand") and kwargs["strand"]:
            if read.strand != transcript.strand: return False
        if read.start < transcript.start or read.stop > transcript.stop :
            return False
        else:
            return compatible(read,transcript,**kwargs)

def translate_fragment_to_transcript_coordinate(read,transcript,**kwargs):
    a=[]
    for i in bam2x.TableIO.parse(read.reads,"bam2bed12",**kwarts):
        a.append(_translate(transcript,i))
    return a
    

def cigar_to_coordinates(cigar,offset=0):
    '''
    demo version
    need to test
    
    deletion from genome (case 3) now consider as exon indel. 
    '''
    exon_starts=[offset]
    exon_lengths=[0]
    state=0
    for i in cigar:
        if i[0]==0 or i[0]==7 or i[0]==8:  # match
           exon_lengths[-1]+=i[1] 
           state=1 
        if i[0]==2:  # deletion from genome , need to consider this should be exon or intron? now count as exon. 
           exon_lengths[-1]+=i[1] 
           state=1 
        if i[0]==3:  # skipped region from the reference 
           if state==1:
               exon_starts.append(exon_starts[-1]+exon_lengths[-1]+i[1]);
               exon_lengths.append(0);
           else:
               exon_starts[-1]+=i[1]
           state=0
    return (exon_starts,exon_lengths)


def parse_string_to_bed(string):
    x=string.split(":")
    if len(x)!=2:
        print >>sys.stderr,"String Format should be\n chromsome:start-end"
        exit(1)
    chr=x[0]
    y=x[1].split("-")
    if len(y)!=2:
        print >>sys.stderr,"String Format should be\n chromsome:start-end"
        exit(1)
    start=int(y[0])-1
    end=int(y[1])
    return BED3(chr,start,end)

def gini_coefficient(iterator):
    '''
    Gini coefficient
    reference:
        http://en.wikipedia.org/wiki/Gini_coefficient
    '''
    l=[i for i in iterator]
    l.sort()
    s1=0.0
    s2=0.0
    y=[]
    n=len(l)
    for i,x in enumerate(l):
        s1+=(i+1)*x
        s2+=x
    G=2*s1/(n*s2)-float(n+1)/n
    return G


def dis2entropy(iterator):
    s=0
    h={}
    for i in iterator:
        if h.has_key(i):
            h[i]+=1
        else:
            h[i]=1
        s+=1
    e=0.0
    for i in h.values():
        f=float(i)/s
        if f!=0.0:
            e-=f*math.log(f)
    return e






def test():
    logging.basicConfig(level=logging.DEBUG)
    a=Bed("chr1",100,200,"a",0.1,"+")
    b=Bed("chr1",200,300,"b",0.2,"+")
    c=Bed("chr1",400,500,"c",0.2,"+")
    d=merge_bed(a,b)
    e=merge_bed(d,c)
    print "E",e
    print "C and B",merge_bed(c,b)
    print "C and B in E",_translate_to_meta(e,merge_bed(c,b))
    print "C and B reverse translate to CHROMOSOME:",reverse_translate(e,_translate_to_meta(e,merge_bed(c,b)))
    print _translate_to_meta(e,e)
if __name__=="__main__":
    test()



import struct
import array
import collections
from bam2x.Turing import  TuringCode
import TuringCodeBook as cb
def write_int_array(ints,f):
    sign=0
    if type(f)==type("str"):
        try:
            f=open(f,"wb")
            sign=1
        except:
            print >>sys.stderr,"can't open file to write"
            exit(0)
    for i in ints:
        f.write(struct.pack('i',i))
    if sign==1:
        f.close()


def read_int_array(f):
    '''
    read any iterable int array or read from file
    '''
    sign=0
    if type(f)==type("str"):
        try:
            f=open(f,"rb")
            sign=1
        except:
            print >>sys.stderr,"can't open file to read"
            exit(0)
    if isinstance(f,file):
        while True:
            a = array.array('i')
            a.fromstring(f.read(4096))
            if not a:
                break
            for i in a:
                yield i
        if sign==1:
            f.close()
        raise StopIteration
    elif isinstance(f,collections.Iterable):
        for i in f:
            yield i
        raise StopIteration

def write_turing_array(turings,f,mode="n"):
    '''
    HEADERCODE: normal 3721
    HEADERCODE: compress 9981
    '''
    if mode=="n":
        header=cb.NORMAL_CODES_ARRAY_HEADER
    elif mode=="c":
        header=cb.COMPRESS_CODES_ARRAY_HEADER

    sign=0
    if type(f)==type("str"):
        try:
            f=open(f,"wb")
            sign=1
        except:
            print >>sys.stderr,"can't open file to write"
            exit(0)
    f.write(struct.pack("i",header))
    if mode=="n":
        for i in turings:
            f.write(struct.pack('i',i.pos))
            f.write(struct.pack('i',i.code))
    elif mode=="c":
        last=turings.next()
        number=1
        for i in turings:
            if i.pos==last.pos and i.code==last.code:
                number+=1
            else:
                f.write(struct.pack('i',last.pos))
                f.write(struct.pack('i',last.code))
                f.write(struct.pack('i',number))
                last=i
                number=1
        f.write(struct.pack('i',last.pos))
        f.write(struct.pack('i',last.code))
        f.write(struct.pack('i',number))
    if sign==1:
        f.close()



def read_turing_array(f,mode="n"):
    '''
    read any turing encode array 
    turing encode array:
      sorting array and has a header
    parse it into NORMAL MODE or COMPRESS MODE
    '''
    r=read_int_array(f)
    header=r.next()
    if mode=="n":
        if header==cb.NORMAL_CODES_ARRAY_HEADER:
            while True:
                try:
                    pos=r.next()
                    code=r.next()
                    yield TuringCode(pos,code)
                except StopIteration:
                    raise StopIteration
        elif header==cb.COMPRESS_CODES_ARRAY_HEADER:
            while True:
                try:
                    pos=r.next()
                    code=r.next()
                    number=r.next()
                    for i in range(number):
                        yield TuringCode(pos,code)
                except StopIteration:
                    raise StopIteration
        raise StopIteration
    elif mode=="c":
        last_pos=None
        last_code=None
        number=None
        if header==cb.COMPRESS_CODES_ARRAY_HEADER:
            while True:
                try:
                    pos=r.next()
                    code=r.next()
                    number=r.next()
                    yield TuringCode(pos,code),number
                except:
                    raise StopIteration
        elif header==cb.NORMAL_CODES_ARRAY_HEADER:
            try:
                last_pos=r.next()
                last_code=r.next()
                number=1
            except StopIteration:
                raise StopIteration
            while True:
                try:
                    pos=r.next()
                    code=r.next()
                    if pos==last_pos and code==last_code:
                        number+=1
                    else:
                        yield TuringCode(last_pos,last_code),number
                        number=1
                        last_pos=pos
                        last_code=code
                except StopIteration:
                    break
            yield TuringCode(last_pos,last_code),number
            raise StopIteration
        raise StopIteration






#!/usr/bin/env python
# Programmer : zhuxp
# Date: 
# Last-modified: 03-17-2014, 15:44:56 EDT
from bam2x import __version__ as VERSION
import pysam
import sys
import gzip
import argparse
suffixToFormat={
    'fa':'fasta',
    'fq':'fastq',
    'genetab':'genebed',
    'bw':'bigwig',
    'tab':'genebed',
    '2bit':'genome'

}

def suffix(string):
    x=string.split('.')
    return x[-1]
from string import lower

def guess_format(string):
    
    x=string.split(".")
    suffix=lower(x[-1])
    if x[-1]=="gz":
        suffix=lower(x[-2])
    if suffixToFormat.has_key(suffix):
        return suffixToFormat[suffix]
    else:
        return suffix

def open_output(output):
    out=None
    if output=="stdout":
        out=sys.stdout
    else:
        try:
            out=open(output,"w")
        except IOError:
            print >>sys.stderr,"can't open file ",output,"to write. Using stdout instead"
            out=sys.stdout
    return out

def open_input(input):
    fin=None
    if input=="stdin":
        fin=sys.stdin
    else:
        try:
            x=input.split(".")
            if x[-1]=="gz":
                fin=gzip.open(input,"r")
            else:
                fin=open(input,"r")
        except IOError:
            print >>sys.stderr,"can't read file",input
            fin=sys.stdin
    return fin
    
def fopen(file,mode="r",**kwargs):
    '''
    '''
    if guess_format(file)=="bam" and mode=="r":
        return pysam.Samfile(file,"rb")
    if mode=="w":
        return open_output(file)
    if mode=="r":
        return open_input(file)
    return None
    
def get_col_num(filehandle,sep="\t"):
    try:
        filehandle.seek(0)
        for i in filehandle:
            if i[0]=="#": continue
            l=i.split(sep)
            filehandle.seek(0)
            return len(l)
    except:
        raise

def read_config(file,sep=":"):
    config={}
    for i in TableIO.parse(file,sep=sep):
        config[i[0]]=i[1]
    return config













def parser_factory(**dict):
    ''' This Function Parse the Argument '''
    group=argparse.ArgumentParser( **dict)
    group.add_argument('-v','--version',action='version',version='%(prog)s '+VERSION)
    group.add_argument('-i','--input',dest='input',default='stdin',type=str,help="input file Default: %(default)s")
    group.add_argument('-o','--output',dest='output',default='stdout',type=str,help="output file Default: %(default)s")
    return group
def mparser_factory(**dict):
    ''' This Function Parse the Argument '''
    group=argparse.ArgumentParser( **dict)
    group.add_argument('-v','--version',action='version',version='%(prog)s '+VERSION)
    group.add_argument('-i','--input',dest='input',default='stdin',type=str,help="input file Default: %(default)s")
    group.add_argument('-o','--output',dest='output',default='stdout',type=str,help="output file Default: %(default)s")
    group.add_argument('-n','--num_cpus',dest='num_cpus',default=4,type=int,help="number of cpus, Default: %(default)i")
    return group

    
if __name__=="__main__":
    pass











#!/usr/bin/env python
# Programmer : zhuxp
# Date: 
# Last-modified: 03-17-2014, 15:15:32 EDT
from __future__ import print_function
import os,sys,argparse
from bam2x import TableIO,Tools
from bam2x import IO
import logging
def set_parser(p):
    ''' This Function Parse the Argument '''
    p.add_argument('-I','--input_format',dest="format",type=str, default="guess", choices=TableIO.hclass.keys(),help="input format")
def help():
    return "sort genome annotation files. input file is genome annotation file."
    
def run(args):
    fin=IO.fopen(args.input,"r")
    out=IO.fopen(args.output,"w")
    if args.format=="guess":
        args.format=IO.guess_format(args.input)
    s=TableIO.parse(args.input,args.format)
    l=[]
    for i,x in enumerate(s):
        if i/10000==0:
            logging.info("reading %s entrys in %s",i,args.input)
        l.append(x)
    logging.info("begin sorting")
    l.sort()
    logging.info("sorting done")
    for i in l:
        print(i,file=out)
    logging.info("completed")

if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())

 




from __future__ import print_function
import os
import sys
import logging
import argparse
from bam2x import TableIO,IO,DBI
from bam2x.Tools import compatible_with_transcript,_translate,reverse_strand,translate_coordinates
import math
import logging
import itertools
import multiprocessing as mp
def help():
    return "aggregation plot, input is bed12 file and bam file, output the aggregation plot number ( and the entropy )"
def set_parser(parser):
    parser.add_argument("-b","--bam",type=str,dest="bam")
    parser.add_argument("-B","--bin_num",type=int,default=100,dest="bin_num")
    parser.add_argument("--bp",type=int,default=100,dest="bp",help="upstream and downstream bp number")
    parser.add_argument("--strand",type=str,choices=["read1","read2"],default="read2",dest="strand",help="strand: read1 or read2")

def output(results,bin_num,gene_num,prefix=""):
    bin_sum=[0 for i in xrange(bin_num)]
    bin_e=[0.0 for i in xrange(bin_num)]
    neg_bin_e=[0.0 for i in xrange(bin_num)]
    neg_bin_sum=[0 for i in xrange(bin_num)]
    bin_dis=[[] for i in xrange(bin_num)]
    neg_bin_dis=[[] for i in xrange(bin_num)]
    bin_dis=[[] for i in xrange(bin_num)]
    for result in results:
        for i in xrange(bin_num):
            bin_sum[i]+=result[0][i]
            bin_dis[i].extend(result[1][i])
            neg_bin_sum[i]+=result[2][i]
            neg_bin_dis[i].extend(result[3][i])
    for i in xrange(bin_num):
        bin_e[i]=dis2entropy(bin_dis[i])
        neg_bin_e[i]=dis2entropy(neg_bin_dis[i])
    for i in xrange(bin_num):
        print("{bin}\t{mean}\t{E}\t{neg_mean}\t{neg_E}".format(bin=prefix+str(i),mean=float(bin_sum[i])/gene_num,E=bin_e[i],neg_mean=float(neg_bin_sum[i])/gene_num,neg_E=neg_bin_e[i]),file=out)

def run(args):
    logging.basicConfig(level=logging.INFO)
    global bam,out
    bam=DBI.init(args.bam,"bam")
    fin=IO.fopen(args.input,"r")
    out=IO.fopen(args.output,"w")
    p=mp.Pool(processes=args.num_cpus)
    beds_list=[[] for i in xrange(args.num_cpus)]
    for i0,bed in enumerate(TableIO.parse(fin,"bed12")):
        beds_list[i0%args.num_cpus].append(bed)
    gene_num=i0+1
    print("bin_id\tmean\tentropy\treverse_strand_mean\treverse_strand_entropy",file=out)
    up_results=p.map(count_flank_star,itertools.izip(beds_list,itertools.repeat(args.bp),itertools.repeat(args.strand),itertools.repeat(True)))
    output(up_results,args.bp,gene_num,"UP")
    results = p.map(count_list_star,itertools.izip(beds_list,itertools.repeat(args.bin_num),itertools.repeat(args.strand)))
    output(results,args.bin_num,gene_num,"TR")
    down_results=p.map(count_flank_star,itertools.izip(beds_list,itertools.repeat(args.bp),itertools.repeat(args.strand),itertools.repeat(False)))
    output(down_results,args.bp,gene_num,"DN")

    

def count_list_star(a_b_c):
    return count_list(*a_b_c)
def count_list(beds,bin_num,strand):        
    bin_sum=[0.0 for i in xrange(bin_num)]
    neg_bin_sum=[0.0 for i in xrange(bin_num)]
    bin_dis=[[] for i in xrange(bin_num)]
    neg_bin_dis=[[] for i in xrange(bin_num)]
    for bed in beds:
        neg_bed=bed._replace(strand=reverse_strand(bed.strand))
        bed_bin=[0.0 for i in xrange(bin_num)]
        neg_bed_bin=[0.0 for i in xrange(bin_num)]
        for read in bam.query(bed,"bam1",strand=strand):
            cdna_length=bed.cdna_length()
            bin_step=float(cdna_length)/bin_num
            if compatible_with_transcript(read,bed):
                gene_bed=_translate(bed,read)
                bin_start=gene_bed.start*bin_num/cdna_length
                bin_stop=(gene_bed.stop-1)*bin_num/cdna_length
                if bin_stop==bin_start:
                    bed_bin[bin_start]+=float(gene_bed.stop-gene_bed.start)/bin_step
                else:
                    bed_bin[bin_start]+=bin_start+1-float(gene_bed.start)/bin_step
                    bed_bin[bin_stop]+=float(gene_bed.stop)/bin_step-bin_stop
                    for i in xrange(bin_start+1,bin_stop):
                        bed_bin[i]+=1.0  
            elif compatible_with_transcript(read,neg_bed):
                gene_bed=_translate(bed,read)
                bin_start=gene_bed.start*bin_num/cdna_length
                bin_stop=(gene_bed.stop-1)*bin_num/cdna_length
                if bin_stop==bin_start:
                    neg_bed_bin[bin_start]+=float(gene_bed.stop-gene_bed.start)/bin_step
                else:
                    neg_bed_bin[bin_start]+=bin_start+1-float(gene_bed.start)/bin_step
                    neg_bed_bin[bin_stop]+=float(gene_bed.stop)/bin_step-bin_stop
                    for i in xrange(bin_start+1,bin_stop):
                        neg_bed_bin[i]+=1.0
        for i in xrange(bin_num):
            bin_sum[i]+=bed_bin[i]
            neg_bin_sum[i]+=neg_bed_bin[i]
            bin_dis[i].append(int(bed_bin[i]))
            neg_bin_dis[i].append(int(neg_bed_bin[i]))
    return bin_sum,bin_dis,neg_bin_sum,neg_bin_dis

def count_flank_star(l):
    return count_flank(*l)
def count_flank(beds,bp,strand,upstream=True):
    '''
    flank seq aggregation ( no splicing)
    if upstream is False , count downstream
    '''
    pos_sum=[0 for i in xrange(bp)]
    pos_dis=[[] for i in xrange(bp)]
    neg_sum=[0 for i in xrange(bp)]
    neg_dis=[[] for i in xrange(bp)]
    for bed in beds:
        if upstream:
            flank=bed.upstream(bp)
            offset=bp-flank.cdna_length()  # in case upstream is less than bp 
        else:
            flank=bed.downstream(bp)
            offset=0
        pos=[0 for i in xrange(bp)]
        neg=[0 for i in xrange(bp)]
        for read in bam.query(flank,"bam1",strand=strand):
            translated_read=translate_coordinates(flank,read)
            if translated_read.strand=="+" or translated_read.strand==".":
                for start,size in itertools.izip(translated_read.blockStarts,translated_read.blockSizes):
                    for j in xrange(start+offset,start+size+offset):
                        if j>=0 and j<bp:
                            pos[j]+=1
            else:
                for start,size in itertools.izip(translated_read.blockStarts,translated_read.blockSizes):
                    for j in xrange(start+offset,start+size+offset):
                        if j>=0 and j<bp:
                            neg[j]+=1
        for i in xrange(bp):
            pos_sum[i]+=pos[i]
            neg_sum[i]+=neg[i]
            pos_dis[i].append(pos[i])
            neg_dis[i].append(neg[i])
    return pos_sum,pos_dis,neg_sum,neg_dis

def dis2entropy(iterator):
    s=0
    h={}
    for i in iterator:
        if h.has_key(i):
            h[i]+=1
        else:
            h[i]=1
        s+=1
    e=0.0
    for i in h.values():
        f=float(i)/s
        if f!=0.0:
            e-=f*math.log(f)
    return e


if __name__=="__main__":
    from bam2x.IO import mparser_factory
    p=mparser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())




# Programmer : zhuxp
# Date: 
# Last-modified: 02-26-2014, 16:51:28 EST
import types
import pysam
from bam2x.Annotation import BED12 as Bed12
from bam2x.Annotation import Fragment
from bam2x import Tools

def BamToBed12(handle,uniq=False,**kwargs):
    '''
    handle is an bam iterator
    need references hash if handle is not filename.
    score="NH" ,will report tag NH as score 
    '''
    if type(handle)==type("string"):
        handle=pysam.Samfile(handle,"rb");
    for i in handle:
        if i.tid<0: continue
        if uniq:
            nh=_get_tag_score(i,"NH")
            if nh:
                if nh > 1: 
                    continue
            else:
                raise "no NH tag in your bam file"
        strand="+"
        if i.is_reverse:
            strand="-"
        score=i.mapq
        
        '''
        test
        '''
        if kwargs.has_key("references"):
            if isinstance(kwargs["references"],str):
                chr=kwargs["references"]  
            else:
                chr=kwargs["references"][i.tid];
        else:
            try:
                 chr=handle.references[i.tid];
            except:
                 chr="chr"
        if kwargs.has_key("strand"):
            if kwargs["strand"]=="read1" or kwargs["strand"]=="firstMate":
                read1=True
            else:
                read1=False
        else:
            read1=True   
        start=i.pos
        end=i.aend
        name=i.qname
        cds_start=start
        cds_end=start
        itemRgb="0,0,0"
        if not uniq:
            '''
            try to put NH score in itemRgb
            '''
            try:
                nh=_get_tag_score(i,"NH")
                if nh:
                    itemRgb=str(nh)+",0,0"
            except:
                pass
        '''
        debug
        import sys
        if i.cigar is None:
            print >>sys.stderr,"why cigar is Nonetype?"
            print >>sys.stderr,i
            exit(0)
        end of debug
        '''
        if i.cigar==None: continue # IGNORE THIS READS?
        (block_starts,block_sizes)=Tools.cigar_to_coordinates(i.cigar);
        if i.is_read1 and not read1:
            strand=Tools.reverse_strand(strand)
        elif i.is_read2 and read1:
            strand=Tools.reverse_strand(strand)
        bed=Bed12(chr,start,end,name,score,strand,cds_start,cds_end,itemRgb,len(block_sizes),block_sizes,block_starts)
        yield bed
def _get_tag_score(read,tag):
    for itag,iscore in read.tags:
        if tag==itag:
            return iscore
    return None


def BamToFragmentIterator(handle,**kwargs):
    '''
    handle is an bamfile or an iterator

    if it is an iterator , 
    a "bam" option could add to find the mate read which are not in the iterator.
    
    The strategy is:
        find all the pairs in iterator first and yield them 
        then
        try to find the rest reads' mate in bamfile.

    One problem is that :
        if too much read are clustered together
        it might be very cost memory if we read too much first read and the iterator still doesn't find their mates.
        TODO: FIX THIS PROBLEM!

    '''
    paired_reads={}
    if type(handle)==type("string"):
        handle=pysam.Samfile(handle,"rb");
    for read in handle:
        fragment_name=strip_mate_id(read.qname)
        if paired_reads.has_key(fragment_name):
            yield Fragment(paired_reads[fragment_name],read) #Paired End fragment
            del paired_reads[fragment_name]
        else:
            if read.is_qcfail or read.is_unmapped:
                continue 
            if  read.mate_is_unmapped or (not read.is_paired):
                yield Fragment(read) # Single end Fragment
            else:
                paired_reads[fragment_name]=read

    
    '''
    the rest of paired end read which haven't find mate yet 
    '''
    db=None
    if kwargs.has_key("bam"): 
        db=kwargs["bam"]
        if type(db)==type("string"):
            db=pysam.Samfile(db,"rb")
    elif isinstance(handle,pysam.Samfile): 
        db=handle
    if db is not None:
        pos=db.tell()
        for read in paired_reads.values():
            try: 
                mate=db.mate(read)
            except ValueError:
                mate=None
                continue
            finally: 
                db.seek(pos)
            yield Fragment(read,mate)
    else:
        for read in paired_reads.values():
            yield Fragment(read)
    del paired_reads


def strip_mate_id(read_name):
    '''
    this function was copied from http://pydoc.net/Python/misopy/0.4.7/misopy.sam_utils/ 
    
    Strip canonical mate IDs for paired end reads, e.g.
    #1, #2
    or:
    /1, /2
    '''
    if read_name.endswith("/1") or read_name.endswith("/2") or read_name.endswith("#1") or read_name.endswith("#2"):
        read_name = read_name[0:-3]
    return read_name





import csv
import bam2x
import sys
from bam2x import IO,Annotation
from bam2x import Translator
hclass = {
    "bed3":Annotation.BED3,
    "bed6":Annotation.BED6,
    "bed12":Annotation.BED12,
    "bed":Annotation.BED6,
    "vcf":Annotation.VCF,
}
htranslate = {
    "bam2bed12": Translator.BamToBed12,
    "bam2fragment":Translator.BamToFragmentIterator,
}
FormatToIterator=dict(hclass.items()+htranslate.items())
def parse(handle,convert_cls=None,**dict):
    if hclass.has_key(convert_cls):
        return parse_tuples(handle,hclass[convert_cls],**dict)
    elif htranslate.has_key(convert_cls):
        return htranslate[convert_cls](handle,**dict)
    elif convert_cls is not None:
        return parse_tuples(handle,convert_cls,**dict)
    else:
        return parse_simple(handle,**dict)

def parse_simple(handle,**dict):
    sep="\t"
    if dict.has_key("sep"):
        sep=dict["sep"]
    if isinstance(handle,str):
        try:
            handle=IO.fopen(handle,"r")
            for i in csv.reader(handle,delimiter=sep):
                if i[0].strip()[0]=="#": continue
                yield tuple(i)
            handle.close()
        except IOError as e:
            print >>sys.stderr,"I/O error({0}): {1}".format(e.errno, e.strerror)
    else:
        try:    
            for i in csv.reader(handle,delimiter=sep):
                if i[0].strip()[0]=="#": continue
                yield tuple(i)
        except:
            raise
    
def parse_tuples(handle,cls,**dict):
    for i in parse_simple(handle,**dict):
        yield cls._make(cls._types(i))


def Main():
    for i in parse(sys.argv[1],"bed12"):
        print i
if __name__=="__main__":
    Main()











#!/usr/bin/env python
# Programmer : zhuxp
# Date: 
# Last-modified: 02-27-2014, 15:45:43 EST
from __future__ import print_function
from distutils.core import setup
from setuptools import setup, find_packages
import codecs
import os
import sys
import re

here = os.path.abspath(os.path.dirname(__file__))

def read(*parts):
    # intentionally *not* adding an encoding option to open
        return codecs.open(os.path.join(here, *parts), 'r').read()

def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                             version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


metadata = {

          'name':"bam2x",
          'version':find_version('lib',"bam2x",'__init__.py'),
          'description':"bionformatics python lib for query bam files, this version set matplotlib as optional.",
          'author':"Xiaopeng Zhu",
          'license':'GNU General Public License',
          'url':'http://github.com/nimezhu/bam2x',
          'author_email':"nimezhu@gmail.com",
          'packages':[
                    "bam2x",
                    "bam2x.DBI",
                    "bam2x.Run",
                    "bam2x.Run.Plot",
                    "bam2x.MRun",
                    "bam2x.Tools",
                    "bam2x.Turing",
                    ],
          'package_dir':{"":"lib"},
          'platforms':'any',
          'scripts':[
                   "bin/bam2x"
                   ],
          'package_data':{"":["README.md"]},
          'install_requires':['numpy>=1.6.0','pysam>=0.7.6','twobitreader>=2.9','bx-python>=0.7.0','bitarray>=0.8.0'],
}

def main():
    if not float(sys.version[:3])>2.4:
        sys.stderr.write("CRITICAL: Python version must be greater than or equal to 2.4!")
        sys.exit(1)
    dist=setup(**metadata)




    
if __name__=="__main__":
    main()








#!/usr/bin/env pythON
# Programmer : zhuxp
# Date: 
# Last-modified: 02-13-2014, 10:03:34 EST
import TuringCodeBook as cb
from bitarray import bitarray 
from TuringUtils import *
from bam2x.Annotation import BED12 as Bed12
__all__=["TuringCode","TuringGraph","TuringSortingArray"]

H_TURING3=("pos","code","cid")
H_TURING2=("pos","code")

from collections import namedtuple


class TuringCode3(namedtuple("TuringCode3",H_TURING3)):
    def __str__(self):
        s=str(self.pos)+":"+str(self.code)+ " cid "+str(self.cid)
        return s
    def __cmp__(self,other):
        return cmp(self.pos,other.pos) or cmp(self.cid,other.cid) or cmp(self.code,other.code)


class TuringCode2(namedtuple("TuringCode2",H_TURING2)):
    def __str__(self):
        s=str(self.pos)+":"+str(self.code)
        return s
    def __cmp__(self,other):
        return cmp(self.pos,other.pos) or cmp(self.code,other.code)

TuringCode=TuringCode2

class TuringGraph:
    def __init__(self,codes,paths=[],**kwargs):
        self.codes=codes
        self.paths=paths
        self.state=cb.BLOCKON
        self.states_register=[]
        self.cid=cb.DEFAULT_CID
        self.codes.sort()
        for key in kwargs.keys():
            setattr(self,key,kwargs[key])
    def set_id(self,id):
        self.id=id
    def get_id(self):
        if self.__dict__.has_key("id"):
            return self.id
        else:
            return "noname"

    def set_cid(self,cid):
        self.cid=cid
        for i in range(len(self.codes)):
            self.codes[i].cid=cid



    def add(path):
        #TO DO
        self.paths.append(path)
        pass
    def __str__(self):
        s=""
        for i in self.codes:
            s+=str(i.pos)+":"+str(i.code)+"\n"
        return s
    def graph_str(self,scale=60):
        s=""
        l=self.codes[-1].pos-self.codes[0].pos
        if scale > l:
            scale=l
        for i in range(scale+1):
            s+="-";
        list_s=list(s)
        
        for i in self.codes:
            position=i.pos-self.codes[0].pos
            index=position*scale/l;
            #print index
            if i.code==cb.BLOCKON and list_s[index]=="-": list_s[index]='^'
            if i.code==cb.BLOCKOFF and list_s[index]=="-" : list_s[index]='v'
            if i.code==cb.BLOCKON and list_s[index]=="v": list_s[index]='x'
            if i.code==cb.BLOCKOFF and list_s[index]=="^": list_s[index]='x'
        return "".join(list_s)
    def path_str(self,path,scale=60):
        s=""
        l=self.codes[-1].pos-self.codes[0].pos
        if scale > l:
            scale=l
        for i in range(scale+1):
            s+=" ";
        list_s=list(s)
        
        for i in path.codes:
            position=i.pos-self.codes[0].pos
            index=position*scale/l;
            print index
            if i.code==cb.BLOCKON and list_s[index]==" ": list_s[index]='^'
            if i.code==cb.BLOCKOFF and list_s[index]==" " : list_s[index]='v'
            if i.code==cb.BLOCKON and list_s[index]==" ": list_s[index]='x'
            if i.code==cb.BLOCKOFF and list_s[index]==" ": list_s[index]='x'
        return "".join(list_s)
 

    def __len__(self):
        if hasattr(self,"_len"):
            return self._len
        d={}
        for i in self.codes:
            if d.has_key(i.pos):
                d[i.pos]+=1
            else:
                d[i.pos]=1
        self._len=len(d.values())
        return self._len
    def score_path(path):
        #TO DO
        pass
    def translate_path_into_bits(self,path,bp=5):
        #TODO: tolerate the reads in +-5bp?
        #TODO  unknown reads if less than given percetage?
        c1=list(self.codes)
        p1=list(path.codes)
        ccid=1
        pcid=2
        c=list()
        for i in c1:
            c.append(TuringCode3(i.pos,i.code,ccid))
        for i in p1:
            c.append(TuringCode3(i.pos,i.code,pcid))
        c.sort()
        cstate=cb.OFF
        cbstate=cb.BLOCKOFF
        pstate=cb.OFF
        pbstate=cb.BLOCKOFF
        #output=bitarray([ True for i in range(2*len(c))])
        output=bitarray(len(self)*2)
        output.setall(True)

        '''
        revise the mis aligned position for path
        '''
        buff=[]
        last_pos=0
        last_code=cb.ON
        for i,x in enumerate(c):
            if c[i].cid==pcid:
                buff.append(i)
            if c[i].cid==ccid:
                for j in buff:
                    if c[j].code%2 ==last_code%2 and abs(c[j].pos-last_pos)<=bp:
                        c[j]=c[j]._replace(pos=last_pos)
                    if c[j].code%2==c[i].code%2 and abs(c[j].pos-c[i].pos)<=bp:
                        c[j]=c[j]._replace(pos=c[i].pos)
                buff=[]
                last_pos=c[i].pos
                last_code=c[i].code
        for j in buff:
            if c[j].code==last_code and abs(c[j].pos-last_pos)<=bp:
                c[j]=c[j]._replace(pos=last_pos)
        '''
        end of revise position
        '''
        j=-1
        last_pos=-1
        #c.sort()
        for i in c:
            #print i.cid,i
            if (i.pos!=last_pos):
                if pstate==cb.ON:
                    if pbstate==cb.BLOCKON:
                        output[2*j]=True;
                        output[2*j+1]=False;
                    if pbstate==cb.BLOCKOFF:
         #               print pbstate,"BLOCKOFF"
                        if output[2*j]==True and output[2*j+1]==True:
                            output[2*j]=False
                elif pstate==cb.OFF:
                    pass
                if(i.cid==ccid):
                    j+=1
                    last_pos=i.pos
            if i.cid==pcid: ## process path
                if i.code==cb.ON: 
                    pstate=cb.ON
                elif i.code==cb.OFF:
                    pstate=cb.OFF
                    
                elif i.code==cb.BLOCKON:
                    pbstate=cb.BLOCKON
                elif i.code==cb.BLOCKOFF:
                    pbstate=cb.BLOCKOFF
            elif i.cid==ccid: ## process self.codes
                if i.code==cb.ON: 
                    cstate=cb.ON
                elif i.code==cb.OFF:
                    cstate=cb.OFF
                elif i.code==cb.BLOCKON:
                    cbstate=cb.BLOCKON
                elif i.code==cb.BLOCKOFF:
                    cbstate=cb.BLOCKOFF
        return output 
    def translate_paths_into_bits(self,paths,bp=5):
        a=[]
        output=bitarray(len(self)*2)
        output.setall(True)
        for i in paths:
            output=bitarray_and(output,self.translate_path_into_bits(i,bp))
        return output
    def translate_bits_into_bed(self,bits,id="noname"):
        chr=self.get_id()
        last_pos=-1
        blockStarts=[]
        blockSizes=[]
        blockCount=0
        itemRgb="0,0,0"
        score=0
        id=id
        i=0
        exon_state=False
        for x in self.codes:
            if x.pos!=last_pos:
                last_pos=x.pos
                #print "debug last pos:",last_pos
                if bits[2*i]==True and bits[2*i+1]==False:
                    if not exon_state:
                        exon_state=True
                        blockStarts.append(last_pos)
                        blockCount+=1
                elif bits[2*i]==False and bits[2*i+1]==True:
                    if exon_state:
                        exon_state=False
                        #print "last pos:",last_pos
                        #print "minus:",blockStarts[-1]
                        blockSizes.append(last_pos-blockStarts[-1])
                elif bits[2*i]==True and bits[2*i+1]==True:
                    if x.code==cb.BLOCKOFF:
                        if exon_state:
                            exon_state=False
                            blockSizes.append(last_pos-blockStarts[-1])
                    if x.code==cb.BLOCKON:
                        if not exon_state:
                            exon_state=True
                            blockStarts.append(last_pos)
                            blockCount+=1
                i+=1
        if blockCount>0:
            start=blockStarts[0]
            stop=blockStarts[-1]+blockSizes[-1]
        else:
            start=0
            stop=0
        cds_start=start
        cds_stop=start
        for i,x in enumerate(blockStarts):
            blockStarts[i]-=start
        strand="+"
        return Bed12(chr,start,stop,id,score,strand,cds_start,cds_stop,itemRgb,blockCount,blockSizes,blockStarts)


    def translate_bits_into_beds(self,bits,id="noname",chr="chr"):
        '''
        beds means to fragmentation the bits.
        not try to link them with missing data
        '''
        chr=chr
        id=id
        last_pos=-1
        blockStarts=[]
        blockSizes=[]
        blockCount=0
        itemRgb="0,0,0"
        score=0
        i=0
        exon_state=False
        i0=0
        strand="+"
        for x in self.codes:
            if x.code!=cb.BLOCKON and x.code!=cb.BLOCKOFF: continue
            if x.pos!=last_pos:
                last_pos=x.pos
                if bits[2*i]==True and bits[2*i+1]==False:
                    if not exon_state and x.code==cb.BLOCKON: # TODO : test if and after is right, debut the bug.bed
                        exon_state=True
                        blockStarts.append(last_pos)
                        blockCount+=1
                elif bits[2*i]==False and bits[2*i+1]==True:
                    if exon_state:
                        exon_state=False
                        blockSizes.append(last_pos-blockStarts[-1])
                elif bits[2*i]==True and bits[2*i+1]==True:
                    if blockCount>0:
                        start=blockStarts[0]
                        if exon_state:
                            exon_state=False
                            blockSizes.append(last_pos-blockStarts[-1])
                        stop=blockStarts[-1]+blockSizes[-1]
                        for j,y in enumerate(blockStarts):
                            blockStarts[j]-=start
                        i0+=1
                        yield Bed12(chr,start,stop,id+"."+str(i0),score,strand,start,start,itemRgb,blockCount,blockSizes,blockStarts)
                        blockCount=0
                        blockStarts=[]
                        blockSizes=[]
                i+=1
    
        if blockCount>0:
            if exon_state:
                blockSizes.append(last_pos-blockStarts[-1])
                exon_state=False
            start=blockStarts[0]
            stop=blockStarts[-1]+blockSizes[-1]
            for j,y in enumerate(blockStarts):
                blockStarts[j]-=start
            i0+=1
            yield Bed12(chr,start,stop,id+"."+str(i0),score,strand,start,start,itemRgb,blockCount,blockSizes,blockStarts)
                    

import array,tempfile,heapq
import itertools
from operator import itemgetter
assert array.array('i').itemsize==4

class TuringSortingArray:
    def __init__(self,a=None,MAX_ARRAY_SIZE=500000,**dict):
        self.data=[[]]
        self.files=[]
        self.size=0
        self.index=0
        self.MAX_ARRAY_SIZE=MAX_ARRAY_SIZE
        if a is not None:
            for i in a:
                self.append(i)
        self.sort()
        self.has_sorted=True
        if dict.has_key("tempdir"):
            self.tempdir=dict["tempdir"]
        else:
            self.tempdir=None
    def append(self,x):
        self.has_sorted=False
        if (self.index<self.MAX_ARRAY_SIZE):
            heapq.heappush(self.data[0],x)
            self.index+=1
        else:
            self.data[0].sort()
            f=tempfile.TemporaryFile(dir=self.tempdir)
            self.data.append(TuringSortingArray.tempfile_reader(f))
            TuringSortingArray.write_tempfile(self.data[0],f)
            f.seek(0)
            self.files.append(f)
            self.data[0]=[x]
            self.index=1
    def sort(self):
        self.data[0].sort()
        self.has_sorted=True
    def seek0(self):
        for f in self.files:
            f.seek(0)
    def iter(self):
        #yield "test"
        if not self.has_sorted:
            self.sort()
            self.has_sorted=True
        self.seek0()
        for i in heapq.merge(*self.data):
            yield i
    @staticmethod    
    def tempfile_reader(f):
        while True:
            a = array.array("i")
            a.fromstring(f.read(4000))
            if not a:
                break
            #print str(f),"DEBUG",len(a)
            for i in range(0,len(a),2):
                yield TuringCode2(a[i],a[i+1])
    @staticmethod
    def write_tempfile(a,f):
        b=array.array("i")
        for i in a:
            b.append(i.pos)
            b.append(i.code)
        b.tofile(f)

    




def test():
    a=list()
    codes=list()
    h={}
    codes.append( TuringCode2(150,cb.BLOCKON))
    codes.append( TuringCode2(1,cb.ON))
    codes.append( TuringCode2(1,cb.BLOCKON))
    codes.append( TuringCode2(100,cb.BLOCKOFF))
    codes.append( TuringCode2(200,cb.BLOCKON))
    codes.append( TuringCode2(300,cb.OFF))
    codes.append( TuringCode2(300,cb.BLOCKOFF))
    codes.sort()

    paths=list()
    paths.append( TuringCode2(1,cb.ON))
    paths.append( TuringCode2(1,cb.BLOCKON))
    paths.append( TuringCode2(100,cb.BLOCKOFF))
    paths.append( TuringCode2(200,cb.BLOCKON))
    paths.append( TuringCode2(300,cb.BLOCKOFF))
    paths.append( TuringCode2(300,cb.OFF))
    g= TuringGraph(codes)
    p= TuringGraph(paths)

    print g;
    print g.graph_str();

    for x in g.codes: print x
    print g.translate_path_into_bits(p)
    print bitarray_to_rep(g.translate_path_into_bits(p))

    print g.translate_bits_into_bed(g.translate_path_into_bits(p))





if __name__=="__main__":
    test()






#!/usr/bin/python
# Programmer : zhuxp
# Date: 
# Last-modified: 02-13-2014, 00:50:47 EST
from matplotlib.backends.backend_pdf import PdfPages
import numpy
import os,sys,argparse
from bam2x import TableIO
import matplotlib
from pylab import *
import pysam 
def help():
    return "draw reads distribution on chromosomes into pdf file"
def set_parser(p):
    ''' This Function Parse the Argument '''
    p.add_argument('-c','--cex',dest="cex",type=float,default=0.5,help="point size default: %(default)f")
    p.add_argument('-b','--binsize',dest="binsize",type=int,default=200,help="binsize default: %(default)i")
    p.add_argument('-l','--logscale',dest="LogScale",action="store_true",default=False,help="binsize default: %(default)s")
def parseIterChrom(fn):
    sam=pysam.Samfile(fn,"rb")
    lengths=sam.lengths
    chrs=sam.references
    for i,chr in enumerate(chrs):
        bins=[0 for row in range(lengths[i]/binsize+1)]
        for a in sam.fetch(chr):
            bins[a.pos/binsize]+=1
        yield chr,bins
    
def run(local_args):
    global args,out,binsize
    args=local_args
    binsize=args.binsize
    pdf=PdfPages(args.output)
    for chrom,bins in parseIterChrom(args.input):
        fig=figure(figsize=(8,6))
        ax=fig.add_subplot(111)
        print >>sys.stderr,"processing",chrom
        pos=[]
        nz_bins=[]
        for j in range(len(bins)):
            if bins[j]==0: continue
            pos.append(j*binsize)
            nz_bins.append(bins[j])
        if args.LogScale:
            plt.plot(pos,numpy.log(numpy.array(nz_bins)),".",markersize=args.cex,color='b')
        else:
            plt.plot(pos,nz_bins,".",markersize=args.cex,color='b')
        title(chrom)
        if chrom=="chr1 " or chrom=="chr1":
            plt.savefig(pdf,format="pdf")
        else:
            pdf.savefig(fig)
        close()
    pdf.close()
    
if __name__=="__main__":
    run()





#!/usr/bin/env python
from __future__ import print_function
# Programmer : zhuxp
# Date: 
# Last-modified: 03-17-2014, 15:04:06 EDT
VERSION="0.0.4"
import os,sys,argparse
from bam2x.Annotation import BED6 as Bed
from bam2x.Annotation import BED12 as Bed12
from bam2x import TableIO,Tools,IO
from bam2x import DBI
import signal
signal.signal(signal.SIGPIPE,signal.SIG_DFL)
import gzip
import time
def help():
    return "cmp region with gene set(bed12 format) "
def set_parser(p):
    ''' This Function Parse the Argument '''
    p.add_argument('-I','--input_format',dest="format",choices=TableIO.hclass.keys(),default="guess",type=str,help="input file format")
    p.add_argument('-g','--gene',dest="genetab",type=str,help="Gene File, BED12 Format ")
    p.add_argument('-u','--upstream',dest="upstream",type=int,default=3000,help="upstream bp number , default: 3000")
    p.add_argument('-d','--downstream',dest="downstream",type=int,default=3000,help="downstream bp number , default: 3000")
    
def run(local_args):
    '''
    IO TEMPLATE
    '''
    global args,out
    args=local_args
    out=IO.fopen(args.output,"w")
    fin=IO.fopen(args.input,"r")
    print("# This data was generated by program ",sys.argv[0]," (version: %s)"%VERSION,file=out)
    print("in bam2x ( https://github.com/nimezhu/bam2x )",file=out)
    print("# Date: ",time.asctime(),file=out)
    print("# The command line is :",file=out)
    print("#\t"," ".join(sys.argv),file=out)
    gene=DBI.init(args.genetab,"binindex",cls="bed12");
    upstream_list=[]
    downstream_list=[]
    exons_list=[]
    introns_list=[]
    utr3_list=[]
    utr5_list=[]
    for g in gene:
        upstream_list.append(g.upstream(args.upstream));
        downstream_list.append(g.downstream(args.downstream));
        for e in g.Exons():
            exons_list.append(e)
        for i in g.Introns():
            introns_list.append(i)
        if not (g.utr3() is None):
            utr3_list.append(g.utr3())
        if not (g.utr5() is None):
            utr5_list.append(g.utr5())
    upstream=DBI.init(upstream_list,"binindex",cls="bed6")
    downstream=DBI.init(downstream_list,"binindex",cls="bed6")
    exons=DBI.init(exons_list,"binindex",cls="bed6")
    introns=DBI.init(introns_list,"binindex",cls="bed6")
    utr3=DBI.init(utr3_list,"binindex",cls="bed6")
    utr5=DBI.init(utr5_list,"binindex",cls="bed6")



    if args.format=="guess":
        args.format=IO.guess_format(args.input)
    for (i0,i) in enumerate(TableIO.parse(fin,args.format)):
        if i0==0:
            if isinstance(i,Bed12):
                print("#chr\tstart\tend\tname\tscore\tstrand\tthick_start\tthick_end\titem_rgb\tblock_count\tblock_sizes\tblock_starts\tgene\tupstream\tdownstream\texon\tintron\tutr3\tutr5",file=out)
            else:
                print("#chr\tstart\tend\tname\tscore\tstrand\tgene\tupstream\tdownstream\texon\tintron\tutr3\tutr5",file=out)

        print(i,file=out,end="")
        print("\t",toIDs(gene.query(i)),file=out,end="")

        print("\t",toIDs(upstream.query(i)),file=out,end="")
        print("\t",toIDs(downstream.query(i)),file=out,end="")
        print("\t",toIDs(exons.query(i)),file=out,end="")
        print("\t",toIDs(introns.query(i)),file=out,end="")
        print("\t",toIDs(utr3.query(i)),file=out,end="")
        print("\t",toIDs(utr5.query(i)),file=out)

def toIDs(x):
    s=""
    for i in x:
            s+=i.id+","
    if s=="":
        s="0"
    return s



if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())

    





